import { CommandHandler } from '@nestjs/cqrs';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { Auth } from '../entities/auth';
import * as bcrypt from 'bcrypt';

@CommandHandler(CreateAuthCommand)
export class CreateAuthUseCase {
  constructor(private readonly _repository: AuthRepository) {}

  async execute(command: CreateAuthCommand): Promise<Auth> {
    const { password, ...authWithoutPassword } = command.createAuthRequest;
    const hash = await bcrypt.hash(password, 10);

    return this._repository.create({
      password: hash,
      ...authWithoutPassword,
    });
  }
}
