import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { UpdateAuthCommand } from '../../commands/update-auth.command';
import { Auth } from '../entities/auth';
import * as bcrypt from 'bcrypt';

@CommandHandler(UpdateAuthCommand)
export class UpdateAuthUseCase {
  constructor(
    private readonly _repository: AuthRepository,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: UpdateAuthCommand): Promise<Auth> {
    const { uuid, username, password } = command.updateAuthRequest;
    const request = {};
    if (username) {
      request['username'] = username;
    }
    if (password) {
      const hash = await bcrypt.hash(password, 10);
      request['password'] = hash;
    }

    return this._repository.update(uuid, request);
  }
}
