import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { CreateAuthRequest } from '../../domain/dtos/create-auth.request';
import { Auth } from '../../domain/entities/auth';
import { CreateAuthUseCase } from '../../domain/usecases/create-auth.usecase';
import { AuthProfile } from '../../mappers/auth.profile';
import * as bcrypt from 'bcrypt';

const newAuthRequest: CreateAuthRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  username: 'john.doe@email.com',
  password: 'John123',
};
const newAuthCommand: CreateAuthCommand = new CreateAuthCommand(newAuthRequest);

const mockAuthRepository = {
  create: jest.fn().mockResolvedValue({
    uuid: newAuthRequest.uuid,
    username: newAuthRequest.username,
    password: bcrypt.hashSync(newAuthRequest.password, 10),
  }),
};

describe('CreateAuthUseCase', () => {
  let createAuthUseCase: CreateAuthUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AuthRepository,
          useValue: mockAuthRepository,
        },
        CreateAuthUseCase,
        AuthProfile,
      ],
    }).compile();

    createAuthUseCase = module.get<CreateAuthUseCase>(CreateAuthUseCase);
  });

  it('should be defined', () => {
    expect(createAuthUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should create an auth and returns new entity object', async () => {
      const newAuth: Auth = await createAuthUseCase.execute(newAuthCommand);

      expect(newAuth.username).toBe(newAuthRequest.username);

      expect(
        bcrypt.compareSync(newAuthRequest.password, newAuth.password),
      ).toBeTruthy();
    });
  });
});
