import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { Auth } from '../../domain/entities/auth';
import * as bcrypt from 'bcrypt';
import { ValidateAuthUseCase } from '../../domain/usecases/validate-auth.usecase';
import { AuthProfile } from '../../mappers/auth.profile';
import { ValidateAuthQuery } from '../../queries/validate-auth.query';

const mockAuthRepository = {
  findOne: jest.fn().mockResolvedValue({
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    username: 'john.doe@email.com',
    password: bcrypt.hashSync('John123', 10),
  }),
};

describe('ValidateAuthUseCase', () => {
  let validateAuthUseCase: ValidateAuthUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AuthRepository,
          useValue: mockAuthRepository,
        },
        ValidateAuthUseCase,
        AuthProfile,
      ],
    }).compile();

    validateAuthUseCase = module.get<ValidateAuthUseCase>(ValidateAuthUseCase);
  });

  it('should be defined', () => {
    expect(validateAuthUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should validate an auth and returns entity object', async () => {
      const auth: Auth = await validateAuthUseCase.execute(
        new ValidateAuthQuery('john.doe@email.com', 'John123'),
      );

      expect(auth.username).toBe('john.doe@email.com');
      expect(bcrypt.compareSync('John123', auth.password)).toBeTruthy();
    });
  });
});
